// file:        ActionIcons.java
// source file: ActionIcons.java
// object file: ActionIcons.class
//
//
// included objects:
//  ActionIcons
//    Desc: implements a Agent specific ActionIcon container/painter
//
//
// history:
//   01.04.97 - first edit version:
//                new functions (drawAction, getIcon, setResources)
//                added DefaultColors-support - Roman
//
//   14.04.97 - first version finished by Roman Valehrach
//
// critical problems: none
//
// known problems: none
//
//
// additional notes:
//   a consuming of the drawn ActionIcon is not trully possible because the 
//   envrionment displays a field multiple times 
//   => only consume after actionOK (Phase 1 =>Phase = 2) and 
//      receivePercepts (Phase 2 =>Phase = 0)
//

import java.awt.*;
import java.awt.image.*;
import java.util.*;


class ActionIcons extends SearchableResources
{
  // private final static boolean DEBUG = false;

  private final static String DEFAULT_GOODICON_COLOR_STR  = "colors.actionicon.GOODCOLOR";
  private final static String DEFAULT_BADICON_COLOR_STR   = "colors.actionicon.BADCOLOR";

  protected ImageReference current_ref_;

  protected String  action_;
  protected int     phase_;
  protected boolean valid_;

  protected Color good_framecolor_;
  protected Color bad_framecolor_;

  private final static String UNKOWN_ACTION = "QUESTION";

// Method: ActionIcons Constructor (public)
//
// *Purpose   : intializes a new instance and defaults to Phase 0
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : only through super
//
  public ActionIcons()
  {
    super();
    consume();             // reset to start-state

    good_framecolor_ = ColorConstants.getColor( defaultGoodIconColorName(), 0x0000ff00);
    bad_framecolor_ =  ColorConstants.getColor( defaultBadIconColorName(), 0x00ff0000 );
  }

  public ActionIcons( Resources images )
  {
    this();
    setResources( images );
  }

// Method: defaultGoodIconColorName (protected)
//         defaultBadIconColorName  (protected)
//
// *Purpose   : returns a default property identifing string for a color
//              - good for beeing overwritten by subclasses to set an other
//                default property identifing string 
//                (the string to be looked up in the system-property list
//
// *Parameters: none
//
// *Return    : String - the new default color
//
// *Variables : all the nessessary id-strings
//
  protected String defaultGoodIconColorName()   { return DEFAULT_GOODICON_COLOR_STR; }
  protected String defaultBadIconColorName()    { return DEFAULT_BADICON_COLOR_STR; }


// Method: setResources (public)
//
// *Purpose   : sets the current action images to the ones provided by
//              the caller - but only if they are unequal null
//              also sets the current_ref-variable to the current image
//              for faster access - threading
//
// *Parameters: actionimages (Resource) - the new action images
//
// *Return    : none
//
// *Variables : 
//
  public void setResources(Resources actionimages)
  {

    if ( (actionimages instanceof Resources ) && 
         (actionimages.images !=null) ) 
    {
      //System.out.println("Setting the currentaction images...");
      setImages( actionimages.getImages() );
      setCheckedDefaultImageReference( UNKOWN_ACTION );
      setDefaultImageReference( UNKOWN_ACTION );

    }

    current_ref_= fetchIconCheck(); // refetch the current image
  }

// Method: getImageReference (private)
//
// *Purpose   : returns the image named by newaction for the current action
//              * if the action is not represented with a image (unknown)
//                a questionmark is displayed
//
// *Parameters: newaction (String) - the name of the action image
//
// *Return    : none
//
// *Variables : multiple ImageReferences...
//
  public ImageReference getImageReference( String newaction )
  {
    if ( usingDefaultImageReference() == false ) 
    {
      setCheckedDefaultImageReference( UNKOWN_ACTION );
//      setDefaultImageReference( UNKOWN_ACTION );
    }

    return searchCheckedImageReference(newaction, true, true ); // refetch the current image
//    return searchImageReference(newaction, true, true ); // refetch the current image

  }// End of getImageReference


// Method: setAction (public)
//
// *Purpose   : sets the current action to be drawn
//              also sets the current_ref-variable to the current image
//              * (null) = no action - no image
//
// *Parameters: action (String) - name of the action icon (^= action)
//              [phase] (int) - current state of the drawingphase (0= invalid)
//
// *Return    : none
//
// *Variables : action (String, class) - holds the current action to be drawn
//
  public void setAction( String action, int phase )
  {
    // System.err.println("setAction: params:>"+action+"</>"+phase+"< action: >"+getAction()+"< (Phase >"+getPhase()+"<");

    if ( action == null )  
    {
      consume();
      return;
    }

    String lastaction = action_;
    action_ = action;
    valid_  = false;
   
    setPhase( phase );

    if ( ( lastaction != null ) &&
         ( action_.equals( lastaction ) ) )
    {
      // equal  => only recheck if last actions fetch-result was no-icon
      if ( current_ref_ == null ) 
        current_ref_= fetchIconCheck();

    } else {   // unequal => always prefetch
      current_ref_= fetchIconCheck();
    }
    // System.err.println("new action: >"+getAction()+"< (Phase >"+getPhase()+"<");
  }

  public void setAction( String action )
  {
    setAction( action, 1);
  }


// Method: getAction (public)
//
// *Purpose   : gets the current action to be drawn
//              (null) = no action
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : action_ (String, class) - holds the current action to be drawn
//
  public String getAction ()
  {
    switch ( phase_ )
    {
      case  0:  return null;
      case  1:  return action_;
      case  2:  return action_;

      default:  consume(); // reset to a valid state
                return action_;
    }
  }


// Method: consume (public)
//
// *Purpose   : deletes the current action to be drawn
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : action_ (String, class) - holds the current action to be drawn
//              phase_ (int, class) - the action icon drawing state
//
  public void consume()
  {
    phase_  = 0;
    action_ = null;
    valid_  = false;

    current_ref_ = null;
  }


// Method: advancePhase (public)
//
// *Purpose   : steps ahead in Phase-State - if possible
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : action_ (String, class) - holds the current action to be drawn
//              phase_ (int, class) - the action icon drawing state
//
  public void advancePhase()
  {
    switch (phase_)
    {
      case 0:  phase_ = 0;
               break;
      case 1:  phase_ = 2;
               break;
      case 2:  consume();
               break;

      default: consume();
    }
  }

  public int getPhase() { return phase_; }

  public boolean setPhase( int phase ) 
  { 
    if ( (phase < 0) || (phase > 2) )
    {
      phase_ = 0;
      return false;
    } else {
      phase_ = phase;
      return true;
    }
  }


// Method: set/getValid (public)
//
// *Purpose   : sets the valid-state of the current action-image 
//              and advances the drawphase (hopefully to 2)
//
//              returns the current validity
//
// *Parameters: boolean - the new action images
//
// *Return    : boolean
//
// *Variables : valid (boolean, class) - is the action (icon) indented valid ?
//              drawphase (int, class) - the action icon drawing state
//
  public void setValid(boolean result)
  {
    valid_ = result;
    advancePhase();
  }

  public boolean getValid() { return valid_; }

// Method: fetchIcon[Check] (public)
//
// *Purpose   : this method fetches the current actions icon from the
//              icons reference and safes it in current_ref_
//
// *Parameters: none
//
// *Return    : none
//
// *Variables : valid_ (boolean, class) - is the action (icon) indented valid ?
//              drawphase (int, class) - the action icon drawing state
//
  public ImageReference fetchIcon()
  {
    ImageReference todraw= null; 
    String drawaction;

    if ( ((drawaction = getAction())              == null) || 
         ( (todraw= getImageReference( drawaction )) == null) )
      return null;

    return todraw;
  }

  public ImageReference fetchIconCheck()
  {
    ImageReference todraw= null; 
    String drawaction;

    if ( ( (drawaction= getAction() )                == null)  || 
         ( (todraw= getImageReference(drawaction) )  == null)  ||
         ( todraw.validImage()                       == false) )
      return null;

    return todraw;
  }

// Method: drawActionFrame (public)
//
// *Purpose   : draws a Frame in the given space - framcolor depends on
//              valid_
//
// *Parameters: g (Graphics) - the graphics to draw the objects to
//              drawArea (Rectangle) - defines the extends of the cross
//
// *Return    : none
//
// *Variables : valid_ (String, class) - holds the current actions result
//              good_framecolor_ (Color, class) - holds the color for OK-Action
//              bad_framecolor_ (Color, class) - holds the color for BAD-Action
//
  public void drawActionFrame(Graphics g, int x, int y, int width, int height) 
  {
    if ( getValid() ) {
      g.setColor( good_framecolor_ );
      // System.err.println( "ActionIcons: drawing good frame");
    }
    else {
      g.setColor( bad_framecolor_ );
      // System.err.println( "ActionIcons: drawing bad frame");
    }

    g.drawRect( x, y, width - 1, height - 1 );
  }

  public void drawActionFrame(Graphics g, Rectangle r) 
  {
    drawActionFrame( g, r.x, r.y, r.width, r.height );
  }

// Method: draw (public)
//
// *Purpose   : draws an action-icon in the upper-right corner of the
//              given rect, the action-icon may fill one forth of the
//              total imagespace (= min-size of the action icon)
//              if bigger the action-icon fill up to one nineth of the
//              field's space
//
// *Parameters: g (Graphics) - the graphics to draw the objects to
//              drawArea (Rectangle) - defines the extends of the cross
//
// *Return    : boolean - was the drawing successful ?
//
// *Variables : action (String) - holds the current action to be drawn
//              drawdim (Dimension) - fourth of the fieldobjects space
//              drawdim3rd (Dimension) - one nineth of the fieldobjects space
//              imagedim (Dimension) - the actual dimensions of the image
//
  public boolean draw(Graphics g, Rectangle drawArea) 
  {
    //ImageReference  todraw = fetchIcon();       // no code change!!
    //  or ...
    //ImageReference  todraw = fetchIconCheck(); // change check below!!
    //  or that way ... must have been fetched with fetchIcon;

    ImageReference todraw = current_ref_; 

    Dimension imagedim;
    Dimension drawdim, drawdim9th;
    Point startpoint;
    boolean loaded = true;

//    int relation = 1; // for now this is fix - could be related to image-dim

    // System.out.println("drawAction called");


    if ( (g == null) || (todraw == null) || (todraw.validImage() == false) )
    {
      // System.err.println("draw ActionIcons: no proper image for action >"+getAction()+"< (Phase >"+getPhase()+"<) ");
      return false; // no framedraw
    } else {
      // System.err.println("draw ActionIcons: trying to paint image");
      loaded = true;
    } 
     
    drawdim = new Dimension();
    drawdim9th = new Dimension();

    imagedim = new Dimension( todraw.image.getWidth(todraw.observer) + 2,
                              todraw.image.getHeight(todraw.observer) + 2);

    if (drawArea.width < drawArea.height) 
    {
      drawdim.width  =  drawArea.width/2;
//      drawdim.height = drawdim.width * relation;
      drawdim.height = drawdim.width;
        drawdim9th.width  =  drawArea.width/3;
      drawdim9th.height = drawdim9th.width;
    } else {
      drawdim.height = drawArea.height/2;
      drawdim.width  = drawdim.height;
      drawdim9th.height = drawArea.height/3;
      drawdim9th.width  = drawdim9th.height;
    }


    if ( loaded == true ) // not loaded ?
    {
      if ( (imagedim.width  < drawdim.width) && 
           (imagedim.height < drawdim.height) ) 
        drawdim = imagedim;

      if ( (imagedim.width  < drawdim9th.width) && 
           (imagedim.height < drawdim9th.height) ) 
        drawdim = drawdim9th;
    }

    if ( (drawdim.width  > drawdim9th.width) || 
         (drawdim.height > drawdim9th.height)) 
      startpoint = new Point( drawArea.x + (drawArea.width - drawdim.width), 
                              drawArea.y );
    else // only if there is enough space (1/9 of the field's space usable)
         // add an extra point distance from the boundrary
      startpoint = new Point( drawArea.x + (drawArea.width - drawdim.width -1), 
                              drawArea.y +1);
                      
    if ( getPhase() == 2 ) 
    {
      drawActionFrame( g,
                       startpoint.x,  startpoint.y,
                       drawdim.width, drawdim.height );
    }

    g.drawImage( todraw.image,
                 startpoint.x   + 1,
                 startpoint.y   + 1,
                 drawdim.width  - 2,
                 drawdim.height - 2,
                 todraw.observer );

    return true;
  }


}//End of class ActionIcons
