// file:        ActiveObject.java
// source file: ActiveObject.java
// object file: ActiveObject.class
//
//
// included objects:
//  ActiveObject
//    Desc: implements a active object for standard environments (no 
//          dimension specified)
//
//
// history:
//   22.02.97 - first version finished by Chris
//
//   13.03.97 - development off. continued by Roman:
//              new extensions for displaying images (as per protocol 11.03.97) - Roman
//   22.03.97 - added & revised documentation - Roman
//   31.03.97 - added new color scheme - Roman
//   08.04.97 - unified Draw (-> super), added getDisplayName -Roman
//   11.04.97 - changed Constructors back to old design - property methods
//              set/getRes and set/getImageObserver will handle optional
//              paramethers now
//
//
// critical problems: none
//
// known problems: none
//
 
import java.awt.*;
import java.awt.image.ImageObserver;
import java.util.*;
import java.lang.*;


abstract class ActiveObject extends FieldObject
{
  protected Vector actions;  //where the actions are stored; though in our examples
                             //there will be only one action at a time
  protected Vector percepts; //where percepts are stored


// Method: ActiveObject Constructor (public)
//
// *Purpose   : intializes a new instance and sets a new image(-resource)
//              for the object
//
// *Parameters: name (String) - name of the new instance
//              type (String) - general type of the new instance
//              resource (String) - name of the resource for the FieldObject
//              observer (ImageObserver) - observer for the FieldObject
//
// *Return    : none
//
// *Variables : actions (Vector, class) - stores the actions
//              percepts (Vector, class) - stores the perceps
//
  public ActiveObject( String name, String type, String resource,
    ImageObserver observer )
  {
    super( name, type, resource, observer );
    actions = new Vector( 1, 3 );
    percepts = new Vector( 7 );
  }

  public ActiveObject( String name, String type, String resource )
  {
    this( name, type, resource, null );
  }

  public ActiveObject( String name, String type,ImageObserver observer )
  {
    this( name, type, null, observer );
  }

  public ActiveObject( String name, String type )
  {
    this( name, type, null, null );
  }

// Method: sendAction (public)
//
// *Purpose   : returns the actions of an ActiveObject
//
// *Parameters: none
//
// *Return    : Vector - the current actions of an ActiveObject instance
//
// *Variables : actions (Vector, class) - stores the actions of an ActiveObject
//
  public Vector sendAction() //where the agent returns his actions
  {
    return( actions );
  }// End of sendAction

 
// Method: receivePercepts (public)
//
// *Purpose   : stores the incoming percepts
//
// *Parameters: perceps (Vector) - the new percept(s) vector
//
// *Return    : none
//
// *Variables : actions (Vector, class) - stores the new percept
//
  public void receivePercepts( Vector percepts )//where the agent receives his percepts
  {
    this.percepts = percepts;
  }// End of receivePercepts


// Method: getDisplayName() (public)
//
// *Purpose   : returns the current name of the object, 
//              * if the type is invalid the type is return
//                and if the type is yet again invalid the string "Me"
//                will be returned
//
// *Parameters: none
//
// *Return    : String - the current name, type or "Me"
//
// *Variables : none
//
   public String getDisplayName()
   {
     String tmpstr = getName();

     if ( tmpstr == null )
     {
       tmpstr = getType();
       if ( tmpstr == null )
         tmpstr = "Me";
     }

     return tmpstr;
   }

// Method: drawPrimitive (public)
//
// *Purpose   : draws a graphic primitive (a default form of displaying)
//              to the graphics device 
//
// *Parameters: g (Graphics) - the graphics to draw the objects to
//              area (Rectangle) - the position and dimensions of the
//                                 object on screen
//              foreground (Color) - the drawingcolor to use
//
// *Return    : whether the drawing of the Image was successfull or not
//
// *Variables : myimage (ImageReference, class) - pointer to a resource
//
  public boolean drawPrimitive( Graphics g, Rectangle area )
  {
    return ( drawPrimitive(g, area, getColor() ) );
  }

  public boolean drawPrimitive( 
    Graphics g, Rectangle area, Color foreground
  )
  {
//    if ( g == null ) return false;

    if ( foreground != null)
      g.setColor( foreground );

    //Primitive: a simple Pylon
    if (area.width < 10) return false;
    if (area.height< 10) return false;

    try {
      Rectangle oval = new Rectangle( area.x, 
                                      area.y, 
                                      (area.width/3),
                                      0 ); // used for temp-storage
      oval.x += (area.width - oval.width)/2;
      oval.height = oval.width/2;
      Polygon body = new Polygon();
      body.addPoint( oval.x + oval.height,    oval.y + oval.height );
      body.addPoint( area.x + 1,             (oval.height = area.y + area.height - 1) );
      body.addPoint( area.x + area.width - 1, oval.height );
      g.fillPolygon( body );
      g.setColor(Color.black);
      g.drawPolygon( body );

      if ( foreground != null) // if no color draw a black-head!
      {
        g.setColor( foreground );
      }

      g.fillOval( oval.x, oval.y, oval.width, oval.width );
      g.setColor(Color.black);
      g.drawOval( oval.x, oval.y, oval.width, oval.width );

    } catch (ArithmeticException e)
    {
      System.err.println("Error calculating (Gold-)Bar: " + e.getMessage());
      return false;
    }

    return true;
  }

}//End of class ActiveObject
