// file:        FieldObject.java
// source file: FieldObject.java
// object file: FieldObject.class
//
//
// included objects:
//  FieldObject
//    Desc: implements a standard FieldObject (drawing functions included)
//
//
// history:
//   22.02.97 - first version finished by Chris
//
//   13.03.97 - development off. continued by Roman:
//              new extensions for displaying images (as per protocol 11.03.97) - Roman
//   22.03.97 - added & revised documentation - Roman
//   05.04.97 - added setRes and getRes - Roman
//   08.04.97 - added setColor/getColor/getColorStr/String2Color - Roman
//   09.04.97 - changed Draw to support added getDisplayName and
//              clipString - Roman
//   11.04.97 - changed Constructors back to old design - property methods
//              set/getRes and set/getImageObserver will handle optional
//              paramethers now - Roman
//
//   14.04.97 - fitted for major redesign - Roman
//
// critical problems: none
//
// known problems:
//   * objects have to have a minimal size -> small objects have problems displaying
//     themselfs
//
 
import java.awt.*;
import java.awt.image.*;
import java.util.*;
import java.lang.*;

import java.util.Properties;

abstract class FieldObject extends Object
{

  private   String name;      // name and type may never change
  private   String type;

  protected String resname;   // default reference to an imagemap
                              // in this example it represents - NO image
 
  public static ResourceLoader resloader;
  public        Resources      resource;  // current reference to be displayed
  public        ImageObserver  observer;

  private final static String DEFAULT_IMAGE = "EMPTY";
  private final static String IMAGE_BASE    = "envs.wumpus.image.";

  private   Color color_;
  private final static String DEFAULT_COLOR_STR = "colors.FIELDOBJECT";

/* for subclasses : reference only
  private final static String DEFAULT_DEATHCOLOR_STR      = "colors.DEATHCOLOR";
  private final static String DEFAULT_CROSSCOLOR_STR      = "colors.CROSSCOLOR";
  private final static String DEFAULT_DEATHCROSSCOLOR_STR = "colors.DEATHCROSSCOLOR";
  private final static String DEFAULT_GOODICON_COLOR_STR  = "colors.GOODACTIONCOLOR";
  private final static String DEFAULT_BADICON_COLOR_STR   = "colors.BADACTIONCOLOR";
*/

// Method: FieldObject Constructor (public)
//
// *Purpose   : initializes the new instance
//
// *Parameters: name (String) - name of the new instance
//              type (String) - general type of the new instance
//              resource (String) - name of the resource for the FieldObject
//              observer (ImageObserver) - observer for the FieldObject
//
// *Return    : none
//
// *Variables : name (String) - name of the new instance
//              type (String) - general type of the new instance
//
//   with set/get-methods:
//              observer (StdReference, class) - pointer to a resource
//              resname (String) - default reference to an imagemap
//
  public FieldObject( String name, String type, String resource,
    ImageObserver observer )
  {
    this.name = name;
    this.type = type;

    if (resloader == null)
      resloader = new ResourceLoader();

    if (resource != null)
      setRes( resource );

    if (observer != null)
      setObserver( observer );

    loadResources();
  }

  public FieldObject( String name, String type, String resource )
  {
    this( name, type, resource, null );
  }
  public FieldObject( String name, String type, ImageObserver observer )
  {
    this( name, type, (String)null, observer );
  }
  public FieldObject( String name, String type )
  {
    this( name, type, (String)null, (ImageObserver)null );
  }

// Method: getName (public, final)
//
// *Purpose   : returns the name of the FieldObject instance
//
// *Parameters: none
//
// *Return    : String - the name of the FieldObject
//
// *Variables : name (String, class) - the instance's name
//
  final public String getName()
  {
    return name;
  }


// Method: getType (public, final)
//
// *Purpose   : returns the general type of the FieldObject instance
//
// *Parameters: none
//
// *Return    : String - the type of the FieldObject
//
// *Variables : type (String, class) - the instance's type
//
  final public String getType()
  {
    return type;
  }


// Method: loadResources (public)
//
// *Purpose   : loads all Resources that can be derived from the 'resname'
//              - resname & ImageObserver must be allready set if differing
//                from default
//              - if getRes() returns null or "EMPTY" the method is not
//                executed
//
// *Parameters: none
//  
// Return     : none
//
// *Variables : resloader (ResourceLoader, class) - the (images-)loader
//              resource (Resources, class ) - the current resources
//
  public void loadResources()
  {
    Image          tmpimg;
    ImageReference tmpref;
    String         fetchres = getRes();

    if ( (fetchres == null) || (fetchres.equals( "EMPTY" )) ) 
      return;

    if (resloader == null)
      resloader = new ResourceLoader();
    
    resource = new DrawableResources();

    tmpimg = resloader.getImage( fetchres, -1, 1 );
    if (tmpimg != null) 
    {
      tmpref   = new StdImageReference( getDisplayName(), tmpimg, getObserver() );
      resource.addImageReference( tmpref );
    }
  }

  public void setResource( Resources newres )
  {
    resource = newres;
  }

  public Resources getResource()
  {
    return resource;
  }

// Method: setRes (public)
//
// *Purpose   : sets the current image's path/name of the FieldObject 
//              instance
//              this method also sets the current default color
//
//              * if param is (null) it searches for the defaultResName()
//               and if that does not help it returns the EMPTY_IMAGE_PROP
//                and if that does not solve it just returns DEFAULT_IMAGE
//
// *Parameters: String - the name of the new image resource
//  
// *Return    : none
//
// *Variables : resname (String, class) - stores the name of the image
//
// *Note      : loadResource() must be called explicitly
//
  public void setRes(String resname)
  {
    if ( resname == null )
    {
      if ( (this.resname = defaultResName()) == null)
      {
//        if ( (this.resname = getPropHandle().getProperty ( "images.EMPTY_IMAGE" ) ) == null );
        {
          // Attention: there may be new environment implementations that can't handle this !!
          //   if so the image image will not be loaded - but getImage checks that
          this.resname = DEFAULT_IMAGE;
        }

        // set the current default color for empty images
        setColor( ( String ) null );
      }
      // System.out.println( getName() + ":Decoded default to :" + this.resname );
    } else {
      // System.out.println( getName() + ": Got the the param image :" + this.resname);
      this.resname = resname;
    }
    // System.out.println( "finished resname... >" + resname + "<");
  }//End of setRes

// Method: getRes (public)
//
// *Purpose   : returns the current image's path/name of the FieldObject 
//              instance
//
// *Parameters: none
//
// *Return    : String - the name of the current image resource
//
// *Variables : resname (String, class) - stores the name of the image
//
  public String getRes()
  {
    if ( this.resname == null ) {
      setRes( null );
      // System.out.println( getName() +": need Image (a Default)..." + this.resname);
    } 
    // else
    //   System.out.println( getName() +": need Image (a set)..." + this.resname);

    if ( this.resname == null)
      return DEFAULT_IMAGE;

     // System.out.println( "Need the image :>"+this.resname_+"<");
    return this.resname;
  }

// Method: defaultResName (public)
//
// *Purpose   : method to select the (property) default resource
//
// *Parameters: none
//
// *Return    : String - the default string to set
//
// *Variables : ret the temporary return string
//
  public String baseResName() { return IMAGE_BASE+getType(); }
  public String defaultResName() 
  {
    ImagePropDecoder decoder= new ImagePropDecoder( baseResName(), null, true, 1, false);
    String ret = decoder.getName();

    if ( ret == null )
    {
      ret = DEFAULT_IMAGE;
      setColor( ( String ) null );
    }
    else
    {
      // System.err.println( getName() + " - colors name is >"+decoder.decodeColor()+"<");
      setColor( decoder.decodeColor() );
    }

    // System.err.println( getName() + " - returning resource >"+getColorStr()+"<");
    return ret;
  }


// Method: defaultRes_Name() (public)
//         defaultColorName  (public)
//
// *Purpose   : returns a default property identifing string for a 
//              resource and color (mostl often the only *NEW* things
//              - good for beeing overwritten by subclasses to set an other
//                default property identifing string 
//                (the string to be looked up in the system-property list
//
//
// *Parameters: none
//
// *Return    : String - the new default resource/color
//
// *Variables : all the nessessary id-strings
//
  public String defaultRes_Name()  { return DEFAULT_IMAGE; }
  public String defaultColorName() { return DEFAULT_COLOR_STR; }

/* for subclasses : reference only
  protected String defaultDeathColorName()      { return DEFAULT_DEATHCOLOR_STR; }
  protected String defaultCrossColorName()      { return DEFAULT_CROSSCOLOR_STR; }
  protected String defaultDeathCrossColorName() { return DEFAULT_DEATHCROSSCOLOR_STR; }
*/

// Method: setColor (public)
//
// *Purpose   : returns the base-color of the current FieldObject
//              * if null WHITE will be the base-color
//
// *Parameters: color (String) - the new base-color
//
// *Return    : none
//
// *Variables : color_ (ColorConstants, class) - the new base-color
//
  public void setColor( String color )
  {
    if ( color_ == null )
    {
      if ( color == null ) 
      {
        color_ = ColorConstants.getColor( defaultColorName() );
      }
      else
      {
        color_ = ColorConstants.getColor( color );
      }
    }
  }

// Method: getColor (public)
//
// *Purpose   : returns the base-color of the current FieldObject
//              * if current color is invalid the Color default will be returned
//
// *Parameters: none or
//              decode (String) - the string to decode to a color
//
// *Return    : Color - the decoded color
//
// *Variables : color_ (ColorConstants, class) - the base-color
//
  public Color getColor()
  {
    if ( color_ == null )
      setColor( (String) null );

    return color_;
  }

// Method: giveImage (public) - obsolete
//
// *Purpose   : sets a new image and/or image-observer
//
// *Parameters: newimg (Image) - the new image
//              newobs (ImageObserver) - a new observer for the image
//
// *Return    : none
//
// *Variables : myimage (StdReference, class) - reference to a resource
//
/* - obsolete - use loadResources()
  public void giveImage( Image newimg, ImageObserver newobs ) 
  {
     // System.out.print("FieldObj. >"+getName()+"< (type:>"+getType()+"<): getting an image");
    if ( ( newimg != null ) || ( newobs != null ) )
    {
      // System.out.println(" - with true reference");
      if ( myimage != null ) 
      {
//        myimage.setName( getDisplayName() );

        if ( newimg != null )
          myimage.setImage( newimg );

        if ( newobs != null )
          myimage.setObserver( newobs );

      } else {
        myimage = new ColoredReference(getDisplayName(), newimg, getColor(), null);
      }
    }
    else
    {
      // System.out.println(" - but no reference ?");
    }
  }
*/

/* - obsolete
  public void giveImage( Image newimg ) 
  {
    giveImage( newimg, ( ImageObserver ) null );
  }// End of giveImage(s)
*/
 
 
/* - obsolete
  public void createEmptyImage()
  // for displaying the Imagename only => no reference to a valid image
  {
    myimage = new ColoredReference(getDisplayName(), null, getColor(), null);
  }
*/
 
// Method: getImage (public) - obsolete
//
// *Purpose   : returns the current image (-resource) for the FieldObject
//              method even checks wheter the image is valid and returns
//              (null) if not
//
// *Parameters: none
//
// *Return    : Image - the current image
//
// *Variables : myimage (StdReference, class) - reference to a resource
//
/* - obsolete - use getResource()
   public Image getImage()
   {
     return myimage.getImage();
   }
*/

// Method: set/getObserver (public)
//
// *Purpose   : sets/returns the observer of the current image (-resource)
//              set method ignores null-argument 
//              (-> use setImageObserverNoCheck or removeImageObserver)
//
// *Parameters: ImageObserver - the new/current observer
//
// *Return    : --#---
//
// *Variables : myimage (StdReference, class) - reference to a resource
//
  public void setObserver( ImageObserver newobs )
  {
    if ( newobs != null ) 
    {
      observer = newobs;
    }
  }

  public void setObserverNoCheck( ImageObserver newobs )
  {
    observer = newobs;
  }

  public ImageObserver getObserver()
  {
    return observer;
  }


// Method: getDisplayName() (public)
//
// *Purpose   : returns the current type of the object, 
//              * if the type is invalid the name is return
//                and if the name is yet again invalid the string "NONE"
//                will be returned
//
// *Parameters: none
//
// *Return    : String - the current type, name or "NONE"
//
// *Variables : none
//
   public String getDisplayName()
   {
     String tmpstr = getType();

     if ( tmpstr == null )
     {
       tmpstr = getName();
       if ( tmpstr == null )
         tmpstr = "NONE";
     }

     return tmpstr;
   }

// Method: drawPrimitive (public)
//
// *Purpose   : draws a graphic primitive (a default form of displaying)
//              to the graphics device 
//
// *Parameters: g (Graphics) - the graphics to draw the objects to
//              area (Rectangle) - the position and dimensions of the
//                                 object on screen
//              foreground (Color) - the drawingcolor to use
//
// *Return    : whether the drawing of the Image was successfull or not
//
// *Variables : myimage (ImageReference, class) - pointer to a resource
//
  public boolean drawPrimitive( Graphics g, Rectangle area )
  {
    return false;
  }

  public boolean drawPrimitive( 
    Graphics g, Rectangle area, Color foreground )
  {
    return false;
  }


// Method: draw (public)
//
// *Purpose   : draws the object on screen - depending on the existence
//              of an imagemap with the image or with it's name (no image)
//
// *Parameters: g (Graphics) - the graphics to draw the objects to
//              drawArea (Rectangle) - the position and dimensions of the
//                                     object on screen
//
// *Return    : none
//
// *Variables : resource - (Resources, class)
//
  public void draw( Graphics g, Rectangle drawArea )
  {
    // System.out.println("FieldObject: "+getName()+" entering Draw function...");
    // System.out.println("FieldObject: params: drawArea.x/y= " +drawArea.x +"/"+ drawArea.y);
    // System.out.println("FieldObject: drawArea.width/height= " +drawArea.width +"/"+ drawArea.height);

    Color   curcol;

    if ( ( resource instanceof DrawableResources ) &&
         (( DrawableResources ) resource).draw( g, drawArea ) )
    {
    }
    else 
      if ( !( drawPrimitive( g, drawArea, curcol = getColor() ) ) &&
           !( StdImageReference.drawString( g, getDisplayName(), drawArea, curcol )) )
      {
        System.err.println("FieldObject: "+getName()+" no way to display myself!");
      }

    // System.out.println("FieldObject >"+getName()+"<: leaving Draw function.");
  }// End of Draw


/*
  public void Draw( Graphics g, Rectangle drawArea )
  {
    draw(g, drawArea);
  }
*/

// Method: finalize (protected)
//
// *Purpose   : only calls the graphics methode g_.finalize,
//              just a small garbage routine
//         
// *Parameters: none
//
// *Return    : none
//
// *Variables : g (graphics, class) - the graphics object of the instance
//
  protected void finalize() throws Throwable
  {
    super.finalize();
  }// End of finalize

}// End of class FieldObject
