// file:        ImageReference.java
// source file: ImageReference.java
// object file: ImageReference.class
//
//
// included objects:
//  ImageReference
//    Desc: implements an reference to a standard image
//
//
// history:
//   07.03.97 - creation of class by Roman Valehrach
//
//   12.03.97 - changes and extensions to class to support environment, 
//              constructors and functions to support a string/image 
//              only reference (observer -> null) - Roman
//   22.03.97 - added & revised documentation - Roman
//   12.04.97 - added methods for image-checking
//
//
// critical problems: none
//
// known problems: none
//

import java.awt.Image;
import java.awt.image.ImageObserver;


class ImageReference extends Object
{
  protected String        name;
  protected Image         image;
  protected ImageObserver observer;

  protected boolean       image_valid; 
    // a state-variable to speed image-checking
    // relying on the current validity (in case of true)
    // there is no need to recheck the image's validtiy

// Method: ImageReference constructor (public)
//
// *Purpose   : initializes the references to name, image and observer
//              with proper values
//
// *Parameters: picname (String) - new name for the pic
//              pic (Image) - new image
//              obsrvr (ImageObserver) - new observer for the pic
//
// *Return    :
//
// *Variables : string (String, class) - stores the new name
//              image (Image, class) - stores the new image
//              observer (ImageObserver, class) - stores the new observer
  public ImageReference( String name, Image image, ImageObserver obsrvr ) 
  {
    this.name=     name;
    this.image=    image;
    this.observer= obsrvr;

    image_valid = checkImage(image, obsrvr);
  }

  public ImageReference( String name, Image image ) 
  {
    this.name=     name;
    this.image=    image;
    this.observer= null;

    image_valid = checkImage( image, null );

  }// End of ImageReference-Constructors


// Method:  setName (public)
//
// *Purpose   : changes the name reference to a new name-string
//              - Save will spare old references if the new one is null
//
// *Parameters: newname (String) - the new Name handle
//         or : none
//
// *Return    : none
//         or : newname (String) - the Name handle
//
// *Variables : string (String, class) - stores the name
//
  public void setName( String newname ) 
  {
    this.name= newname; 
  }

  public void setNameSave( String newname ) 
  {
    if (newname != null)
      this.name= newname; 
  }

  public static String getName( ImageReference ref ) 
  {
    if (ref == null)
      return null;
    return ref.name; 
  }

  public String getName() 
  {
    return this.name; 
  }//End of get/sets

// Method:  set/getImage (public)
//
// *Purpose   : changes the Image reference to a new image
//              - Save will spare old references if the new one is null
//              - Checked will only set an Image that is valid (as per checkImage)
//
//              all sets (except NoCheck(s)) will perform a validiy check 
//              on the current image
//
// *Parameters: newimage (Image) - the new Image handle
//         or : none
//
// *Return    : none
//         or : newimage (Image) - the Image handle
//
// *Variables : image (Image, class) - stores the new image
//
  public void setImage( Image newimage ) 
  {
    this.image= newimage; 
    checkImage();
  }

  public void setImageNoCheck( Image newimage ) 
  {
    this.image= newimage; 
    image_valid = false;
  }

  public void setImageSave( Image newimage ) 
  {
    if (newimage != null)
    {
      this.image= newimage; 
      checkImage();
    }
  }

  public void setImageSaveNoCheck( Image newimage ) 
  {
    if (newimage != null)
    {
      this.image= newimage; 
      image_valid = false;
    }
  }

  public void setCheckedImage( Image newimage ) 
  {
    boolean tmp_valid;
    if ( (tmp_valid = checkImage(newimage, getObserver())) )
    {
      this.image= newimage; 
      image_valid = true;
    }
  }

  public Image getImage() 
  {
    return this.image; 
  }

  public static Image getImage( ImageReference ref ) 
  {
    if (ref == null) return null;

    return ref.image; 

  }//End of get/sets


// Method:  set/getObserver (public)
//
// *Purpose   : changes the ImageObserver reference to a new image 
//              - Save will spare old references if the new one is null
//
// *Parameters: newobserver (ImageObserver) - the new ImageObserver handle
//         or : none
//
// *Return    : none
//         or : newobserver (ImageObserver) - the ImageObserver handle
//
// *Variables : observer (ImageObserver, class) - stores the observer
//
  public void setObserver( ImageObserver newobserver ) 
  { 
    this.observer= newobserver; 
  }

  public void setCheckedObserver( ImageObserver newobserver ) 
  {
    if (newobserver != null)
      this.observer= newobserver; 
  }

  public ImageObserver getObserver() 
  { 
    return this.observer; 

  }//End of get/sets


  public static ImageObserver getImageObserver( ImageReference ref ) 
  {
    if (ref == null)
      return null;
    return ref.observer; 
  }


// Method: checkImageReference (public, static)
//
// *Purpose   : checks wheter the current reference is valid and
//              returns the result
//
// *Parameters: image (Image) - the image to check
//         or : observer (ImageObserver) - the imageobserver to appley 
//              (or null)
//
//        or  : none
//
// *Return    : boolean - the result of the check
//
// *Variables : none
//
  public static boolean checkImageReference( ImageReference img_ref )
  {
    return checkImage( img_ref );
  }

  public boolean checkImageReference()
  {
    return validImage();
  }


// Method: doImageReferenceCheck (public, static)
//
// *Purpose   : checks wheter the current reference is valid and
//              returns the image or null depending on the result
//
// *Parameters: image (Image) - the image to check
//         or : observer (ImageObserver) - the imageobserver to appley 
//              (or null)
//
//        or  : none
//
// *Return    : Image - the checked Image
//
// *Variables : none
//
  public static ImageReference doImageReferenceCheck( ImageReference img_ref )
  {
    if ( checkImage( img_ref ) )
      return img_ref;
    else
      return null;
  }

  public ImageReference doImageReferenceCheck(  )
  {
    if ( checkImage() )
      return this;
    else
      return null;
  }


// Method: checkImage (public, static)
//
// *Purpose   : checks wheter the current reference/image is valid and
//              returns the result
//
// *Parameters: image (Image) - the image to check
//         or : observer (ImageObserver) - the imageobserver to appley 
//              (or null)
//
//        or  : none
//
// *Return    : boolean - the result of the check
//
// *Variables : none
//
  public static boolean checkImage( Image image, 
                                    ImageObserver observer )
  {
    if ( ( image == null ) ||
         ( image.getWidth(  observer ) < 0 ) ||
         ( image.getHeight( observer ) < 0 ) )
      return false;
    else
      return true;
  }

  // check image with NULL observer....
  public static boolean checkImage( Image image )
  {
    return checkImage( image, null );
  }

  public boolean checkImage()
  {
    Image         image = getImage();
    ImageObserver observer;

    // mirror's doImageCheck code for speed....
    if ( ( image == null ) ||
         ( image.getWidth ( (observer=getObserver()) ) < 0 ) ||
         ( image.getHeight(  observer                ) < 0 ) )
    {
      return (image_valid = false);
    }
    else
    {
      return (image_valid = true);
    }
  }

  // check a reference, needed only in 'special' cases
  public static boolean checkImage( ImageReference image_ref )
  {
    if ( image_ref == null )
    {
      Image         image = image_ref.getImage();
      ImageObserver observer;

      // mirror's doImageCheck code for speed....
      if ( ( image == null ) ||
           ( image.getWidth ( (observer=image_ref.getObserver()) ) < 0 ) ||
           ( image.getHeight(  observer                          ) < 0 ) )
      {
        return (image_ref.image_valid = false);
      }
      else
      {
        return (image_ref.image_valid = true);
      }
    }
    else
      return false;
  }

// Method: doImageCheck (public, static)
//
// *Purpose   : checks wheter the given image is valid and
//              returns this image or null depending on the result
//
// *Parameters: image (Image) - the image to check
//         or : observer (ImageObserver) - the imageobserver to appley 
//
// *Return    : Image - the checked Image
//
// *Variables : none
//
  public static Image doImageCheck( Image image, 
                                    ImageObserver observer )
  {
    if ( ( image == null ) ||
         ( image.getWidth(  observer ) < 0 ) ||
         ( image.getHeight( observer ) < 0 ) )
      return null;
    else
      return image;
  }

  // check image with NULL observer....
  public static Image doImageCheck( Image image )
  {
    return doImageCheck( image, null );
  }

// Method: doImageCheck (public, static)
//
// *Purpose   : checks wheter the current image is valid and
//              returns the image or null depending on the result
//
// *Parameters: none
//
// *Return    : Image - the checked Image
//
// *Variables : image_valid (booldean, class) - a state-variable to speed image drawing
//
  public Image doImageCheck()
  {
    Image         image = getImage();
    ImageObserver observer;

    // mirror's checkImage code for speed....
    if ( ( image == null ) ||                  
         ( image.getWidth ( (observer=getObserver()) ) < 0 ) ||
         ( image.getHeight(  observer                ) < 0 ) )
    {
      image_valid = false;
      return null;
    }
    else
    {
      image_valid = true;
      return image;
    }
  }

  // this is only needed in 'special' cases
  public static Image doImageCheck( ImageReference image_ref )
  {
    if (image_ref != null)
    {
      Image         image = image_ref.getImage();
      ImageObserver observer;

      // mirror's checkImage code for speed....
      if ( ( image == null ) ||                  
           ( image.getWidth ( (observer=image_ref.getObserver()) ) < 0 ) ||
           ( image.getHeight(  observer                          ) < 0 ) )
      {
        image_ref.image_valid = false;
        return null;
      }
      else
      {
        image_ref.image_valid = true;
        return image;
      }
    }
    else 
     return null;
  }


// Method: validImage (public)
//
// *Purpose   : checks wheter the current image is valid
//              - only rechecks in case of false
//
// *Parameters: none
//
// *Return    : boolean - is the image valid ?
//
// *Variables : image_valid (booldean, class) - a state-variable to speed 
//              image drawing
//
// *add. Note : this method speeds up image-checking in case
//              one can trust a subclasses - implementation of 
//              the subclasses constructor, setImage, checkImage, etc.
//              (they really set image_valid properly..)
//
  public boolean validImage()
  {
    if ( !image_valid )
      checkImage();

    return image_valid;
  }


}// End of class ImageReference
