// file:        ResourceLoader.java
// source file: ResourceLoader.java
// object file: ResourceLoader.class
//
//
// included objects:
//  ResourceLoaders
//    Desc: implements a class that loads all images for FieldObjects
//          (Actions are loaded through it's own loader)
//
// history:
//   17.04.97 - first version finished by Roman Valehrach
//
//
// critical problems: none
//
// known problems: none
//

import java.awt.*;
import java.awt.image.*;
import java.util.*;
import java.lang.*;
import java.net.*;
import java.io.*;
// import sun.applet.*;
import java.applet.Applet;


class ResourceLoader
{
  // private static final boolean DEBUG = false;

  protected static DefaultProps        props_;
  protected static Applet              app;

  protected static URL                 baseurl_;
  protected static Canvas              canvas_;
  protected static MediaTracker        tracker_;
  protected static ImageObserver       myobs_;

  protected static int                 currentTID_;
  protected static boolean             increasedTracker_;

  protected static SearchableResources res_;
  protected static ActionImagesLoader  actionimages_;
  protected static StoreList           checkList_;

  protected static final int    MAX_TRY           = 10;


// Method: ResourceLoader Constructor (public)
//
// *Purpose   : intializes a new instance and sets a new image (-resource) 
//              for the object
//              - a MediaTracker will be created automatically
//               (if none is given or if the imageOffset is not -1)
//
// *Parameters: baseurl (URL/String) - the basic url of the current browser
//              imgobs (ImageObserver) - an observer for all images
//              trackerStartID (int) - offset/startnumber for the tracker
//              increaseTracker (boolean) - whether to increase the Tracker
//                                          after every new image created ?
//
// *Return    : none
//
// *Variables : class-global only
//
  public ResourceLoader()
  {
    if (res_ == null) 
      res_ = new SearchableResources();

    // !Attention!: World or someone else must initialize with 
    //              -> new ResourceLoader( URL ); <-
  }

  public ResourceLoader( String baseurl )
  {
    this( baseurl, null, null, 1, false );
  }

  public ResourceLoader( URL baseurl )
  {
    this( baseurl, null, null, 1, true );
  }


  public ResourceLoader( 
    URL           baseurl, 
    ImageObserver imgobs, 
    MediaTracker  tracker,
    int           trackerStartID,
    boolean       increaseTracker
  )
  {
    super();

    if (res_ == null) 
      res_ = new SearchableResources();

    baseurl_          = baseurl;
    myobs_            = imgobs;
    app               = null;

    if (trackerStartID != -1)
    {
      if (tracker == null)
      {
        canvas_= new Canvas();
        tracker_= new MediaTracker(canvas_);
      } else {
        canvas_ = null;
        tracker_= tracker;
      }
    }

    setTrackID( trackerStartID, increaseTracker );

    if (props_ == null)
    {
      Properties prop = DefaultProps.getDefaultProperties();

      if ( prop == null ) {  // DefaultProperties initialized ?
        // System.out.println("Trying to set up the Applet-Properties");
        props_= new DefaultProps( baseurl_ );
      }
      else {
        // System.out.println("Applet-Properties allready set...");
        props_= new DefaultProps();
      }
    }
  }

  public ResourceLoader(             // this one is only in case the baseurl (as a URL-instance) is not needed
    String        baseurl, 
    ImageObserver imgobs, 
    MediaTracker  tracker,
    int           trackerStartID,
    boolean       increaseTracker
  )
  {
    super();

    if (res_ == null) 
      res_ = new SearchableResources();

    myobs_            = imgobs;
    app               = null;

    // System.err.println(" System is: " +System.getProperty("os.name"));

    if (trackerStartID != -1)
    {
      if (tracker == null)
      {
        canvas_= new Canvas();
        tracker_= new MediaTracker(canvas_);
      } else {
        canvas_ = null;
        tracker_= tracker;
      }
    }

    setTrackID( trackerStartID, increaseTracker );

    try
    {
      baseurl_ = new URL( baseurl );

      if (props_ == null)
      {
        Properties prop = DefaultProps.getDefaultProperties();

        if ( prop == null ) // DefaultProperties initialized ?
        {
          // System.out.println("Trying to set up the Applet-Properties");
          props_= new DefaultProps( baseurl_ );
        }
        else
        {
          // System.out.println("Applet-Properties allready set...");
          props_= new DefaultProps();
        }
      }
    } 
    catch (MalformedURLException e1)
    {
      System.err.println("ResourceLoader (constructor) URL-Creation Error: " + e1.getMessage());
      e1.printStackTrace();
    }

  }

  public ResourceLoader( 
    String        baseurl, 
    ImageObserver imgobs, 
    int           trackerStartID,
    boolean       increaseTracker
  )
  {
    this( baseurl, imgobs, null, trackerStartID, increaseTracker );
  }

  public ResourceLoader( 
    URL           baseurl, 
    ImageObserver imgobs, 
    int           trackerStartID,
    boolean       increaseTracker
  )
  {
    this( baseurl, imgobs, null, trackerStartID, increaseTracker );

  } // end of Constructors


// Method: setTrackID (public)
//
// *Purpose   : sets the Track-ID for the next image and
//              wheter this ID shall be increased for the
//              following images
//
// *Parameters: startID (int) - start-ID for the tracker
//              increaseTracker (boolean) - whether to increase the Tracker
//
// *Return    : void
//
// *Variables : currentTID_ (int, class) - the ID for the next image
//              increasedTracker_ (boolean, class) - whether to increase 
//                the Tracker
//
  public void setTrackID( int startID, boolean increaseTracker )
  {
    increasedTracker_ = increaseTracker;
    currentTID_       = startID;
  }

  public void setTrackID( int startID )
  {
    currentTID_       = startID;
  }


// Method: doFilter (public, static)
//
// *Purpose   : filters an image
//
// *Parameters: orig_img (Image) - the image to start with
//              fltr (ImageFilter) - the filter to use on the image
//
// *Return    : Image - a new highlightend Image-instance
//
// *Variables : highltn (Image) - the new image
//
  public static Image doFilter( Image orig_img, ImageFilter fltr, Applet app )
  {
    Image fltrd = null;

    if ( ( orig_img != null ) && (fltr != null ) )
    {
      try 
      { 
        ImageProducer producer;
        producer = new FilteredImageSource( orig_img.getSource(), fltr );

        if (app == null)
        {
          Toolkit toolkit = Toolkit.getDefaultToolkit();
          fltrd = toolkit.createImage( producer );
        } else {
          fltrd = app.createImage( producer );
        } 
      } 
//      catch ( InterruptedException e ) 
//      {
//      }
      catch ( Exception e ) // catch all
      {
      }

    }

    return fltrd;
  }


// Method: doFilterStartLoad (public, static)
//
// *Purpose   : filters an image and adds it to a mediatracker
//
// *Parameters: orig_img (Image) - the image to start with
//              fltr (ImageFilter) - the filter to use on the image
//              tracker (MediaTracker) - the tracker to watch the loading
//              trackid (int) - the number the tracker shall id with
//
// *Return    : Image - a new highlightend Image-instance
//
// *Variables : highltn (Image) - the new image
//
  public static Image doFilterStartLoad ( 
   Image orig_img, ImageFilter fltr, MediaTracker tracker, int trackid )
  {
    Image fltrd = null;

    if ( ( orig_img != null ) && (fltr != null ) )
    {
      try 
      { 
        ImageProducer producer;
        producer = new FilteredImageSource( orig_img.getSource(), fltr );

        Toolkit toolkit = Toolkit.getDefaultToolkit();
        fltrd = toolkit.createImage( producer );

        if (tracker != null)
          tracker.addImage( fltrd, trackid );
      } 
//      catch ( InterruptedException e ) 
//      {
//      }
      catch ( Exception e ) // catch all
      {
      }

    }

    return fltrd;
  }


// Method: doFilterNload (public, static)
//
// *Purpose   : filters an image and loads it with a mediatracker at once
//
// *Parameters: orig_img (Image) - the image to start with
//              fltr (ImageFilter) - the filter to use on the image
//              tracker (MediaTracker) - the tracker to watch the loading
//              trackid (int) - the number the tracker shall id with
//
// *Return    : Image - a new highlightend Image-instance
//
// *Variables : highltn (Image) - the new image
//
  public static Image doFilterNload( Image orig_img, ImageFilter fltr, 
                                     MediaTracker tracker, int trackid )
  {
    Image fltrd = null;

    if ( ( orig_img != null ) && (fltr != null ) )
    {
      try 
      { 
        ImageProducer producer;
        producer = new FilteredImageSource( orig_img.getSource(), fltr );

        Toolkit toolkit = Toolkit.getDefaultToolkit();
        fltrd = toolkit.createImage( producer );

        if (tracker != null)
        {
          tracker.addImage( fltrd, trackid );

          tracker.waitForID( trackid ); 
        }
      } 
      catch ( InterruptedException e ) 
      {
      }
      catch ( Exception e ) // catch all
      {
      }

      // Check for errors loading the image.
      if ( (tracker != null) && tracker.isErrorID( trackid ) ) {
        // System.err.println( "Error filtering an image!" );
      }    
    }

    return fltrd;
  }


// Method: doHighlighten (public, static)
//
// *Purpose   : highlightens (filters) an image
//
// *Parameters: orig_img (Image) - the image to start with
//       optional: (all or none)
//              fltr (ImageFilter) - the filter to use on the image
//              tracker (MediaTracker) - the tracker to watch the loading
//              trackid (int) - the number the tracker shall id with
//
// *Return    : Image - a new highlightend Image-instance
//
// *Variables : highltn (Image) - the new image
//
  public static Image doHighlighten( Image orig_img )
  {
    if ( orig_img != null )
    {
      Canvas canvas = new Canvas();
      MediaTracker tracker = new MediaTracker( canvas );
      ImageFilter fltr = new GrayFilter();

      return doFilterNload( orig_img, fltr, tracker, 1);
    } 
    else
      return null;
  }

// Method: doHighlighten (public, static)
//
// *Purpose   : filters an image
//
// *Parameters: orig_img (Image) - the image to start with
//              fltr (ImageFilter) - the filter to use on the image
//
// *Return    : Image - a new highlightend Image-instance
//
// *Variables : highltn (Image) - the new image
//
  public static Image doHighlighten( Image orig_img, MediaTracker tracker, 
    int trackid ) 
  {
    if ( orig_img instanceof Image )
    {
      ImageFilter fltr = new GrayFilter();
      return doFilterNload(orig_img, fltr, tracker, trackid );
    }
    else
      return null;
  }

  public static Image doTransparency( Image orig_img )
  {
    if ( orig_img instanceof Image )
    {
      Canvas canvas = new Canvas();
      MediaTracker tracker = new MediaTracker( canvas );
      ImageFilter fltr = new TransparencyFilter();

      return doFilterNload( orig_img, fltr, tracker, 1);
    } 
    else
      return null;
  }

  public static Image doTransparency( Image orig_img, MediaTracker tracker, 
    int trackid ) 
  {
    if ( orig_img instanceof Image )
    {
      ImageFilter fltr = new TransparencyFilter();
      return doFilterNload(orig_img, fltr, tracker, trackid );
    }
    else
      return null;
  }

//-------------------
/*
  protected FieldObject[] createFieldBase ( Field fld, int[] hastype )
  {
    String type = fld.getType();
    FieldObject[] fieldbases = null;
    FieldObject   tmpbase = null;


    if ( (hastype == null) || (hastype.length < 1) )
      return null;

    int i = 0;

    if ( type.equalsIgnoreCase("WALL") )
    {
      fieldbases = new FieldObject[1];
      fieldbases[i] = new Wall();
      hastype[0] = 1;
    }
    else
    if ( type.equalsIgnoreCase("EXIT") )
    {
      fieldbases = new FieldObject[1];
      fieldbases[i] = new Exit();
      hastype[0] = 2;
    }
    else
    if ( type.equalsIgnoreCase("PIT") )
    {
      fieldbases = new FieldObject[1];
      fieldbases[i] = new Pit();
      hastype[0] = 4;
    }

    if (fieldbases != null)
    {
      while ( i < fieldbases.length )
      {
        fld.addObjectNoCheck( fieldbases[i++] );
      }
    }

    return fieldbases;
  }

  public ImageReference basicReference( int type )
  {
    switch ( type & 0x00ff )
    {
      case 0x01:  // just a box filling all
        return new PrimitivesReference( null, null, myobs_, null, null );

      case 0x04:  // if possible - a box filling 2/3 of the space
        return new SmallBoxReference( null, null, myobs_, null, null );

      case 0x20: // a Pylon-like Figure with a box top
        return new BoxHeadReference( null, null, myobs_, null, null );

      case 0x21: // (Gold-)Bar
        return new BarReference( null, null, myobs_, null, null );

      case 0x40: // a Pylon-like Figure with a round top
        return new PylonReference( null, null, myobs_, null, null );

      case 0x80: // a Pylon-like Figure with a round top
        return new PylonReference( null, null, myobs_, null, null );

      default:
        return new ColoredReference( null, null, myobs_, null, null );
    }
  }
*/

  public Resources getActionResources()
  {
    // System.out.println("trying to get Action-Images-Resource");

    if ( actionimages_ == null )
      actionimages_ = new ActionImagesLoader( myobs_ );

    return (Resources)actionimages_;
  }


// Method: getResource (public)
//
// *Purpose   : gets all possible/needed Action-Images
//
// *Parameters: obj (Object) - the Object that needs a resource
//              envparam (String) - the HTML-page parameters for the Object
//
// *Return    : Resources - the resources given to the Object
//
// *Variables : images (Vector, class) - the current image-resources
//
/*
  public Resources getResource( Object obj, String envparam )
  {
    Image tmpimg=null;
    ImageReference tmpref;
    String searchstr = null;

    String baseRes = null;
    String dispstr = null;

    Resources newRes = new DrawableResources();
    FieldObject[] fldobjs= null;
    FieldObject   currobj= null;
    Object[][] param = null;
    int max = 1;
    int type = 0;
    int[] Itype = {0};
    boolean killable = false;

    int i = 0;
    int[]   fltrType = new int[1];
    Color[] tmpColor = null;

    if ( obj instanceof Field )
    {
      fldobjs = createFieldBase( (Field) obj, Itype );

      if ( (fldobjs == null) || (fldobjs.length < 1) )
        return null;
      else
      {
        if ( (currobj = fldobjs[0]) == null )
          return null;

        type = Itype[0];
        if (type == 0x01)  fltrType[0] = 0;
        else               fltrType[0] = 1;

        dispstr = currobj.getType();
      }
    }
    else if ( obj instanceof FieldObject ) 
    {
      currobj = (FieldObject) obj;
      if ( currobj instanceof PassiveObject )
      {
        if ( currobj instanceof Wumpus )
        { type = 0x8020; max++; } 
        else if ( currobj instanceof Gold )
        { type = 0x0021; }

        fltrType = new int[2];
        tmpColor = new Color[2];
        fltrType[0] = 1;
        fltrType[1] = 102;
        tmpColor[1] = ColorConstants.getColor( "colors.fieldobject.DEATHCOLOR" );
      }
      else if ( currobj instanceof ActiveObject )
      {
        dispstr = currobj.getName();
        if ( currobj instanceof WumpusAgent )
        { 
          type = 0x8040; 
          max++; 
        }
        else if ( currobj instanceof VacuumAgent )
        { 
          type = 0x8080; 
          max++; 
        }
        fltrType = new int[2];
        tmpColor = new Color[2];
        fltrType[0] = 1;
        fltrType[1] = 102;
        tmpColor[1] = ColorConstants.getColor( "colors.fieldobject.DEATHCOLOR" );
      }
      else // every other form of FieldObject
      {
        dispstr = currobj.getType();

        fltrType[0] = 1;
      }
    }

    if (tmpColor == null)
      tmpColor = new Color[1];
    tmpColor[0] = currobj.getColor();
  
    baseRes = currobj.getRes();
    searchstr = baseRes;

//    if ( DEBUG )
//      System.out.print("Field/Object:" + dispstr + " needs a resource... for: >");
    
    if ( ( envparam != null ) && !envparam.equals( "EMPTY" ) )
    { 
      searchstr = envparam;
    } else {
      if ( (searchstr == null) || (searchstr.equals("EMPTY")) )
      {
        searchstr = null;
      }
    }


    while ( (fltrType != null) && 
            (i < fltrType.length) ) // currently only one/two
    {
//      if ( DEBUG )
//        System.out.println( searchstr + "<....");

      tmpimg = getImage( searchstr, -1, fltrType[i], null );

      tmpref = basicReference( type );

      if ( tmpref != null )
      {
        tmpref.setName( dispstr );
        tmpref.setImage( tmpimg );

        if (tmpref instanceof ColoredReferenceIfc)
          ((ColoredReferenceIfc)tmpref).setColor( tmpColor[i] );

        newRes.addImageReference( tmpref );
      }
 
      i++;
    }

    currobj.setResource(newRes);
    return newRes;
  } // End getResource()
*/

// Method: getImage (public)
//
// *Purpose   : fetches an Image from the current list or creates it
//              if it doesn't exit in list (Vector Images)
//
// *Parameters: search (String) - the URL of the image needed
//              
//              trackID (int) - a ID-number for a possible creation
//                if this is -1 the current number of the Loader will
//                be used
//              
//              fltrType (int) - the id for the filter to use (0 ^= none)
//                if fltType (>100) the image filterd/100 will be used
//                as the base of the image
//              
//              fltr (ImageFilter) - a new Filter that is not included
//                in the current implementation - fltrType will be used to 
//                reference the new image in the Resource-Library
//
//              
// *Return    : Image - a new highlightend Image-instance
//
// *Variables : images (Vector, class) - the current image-resources
//              baseurl_(URL, class) - the basic URL for loading
//              tracker_(MediaTracker, class) - for tracking the loading
//              checkList_ (StoreList, class) - stores all the unchecked images
//              currentTID_ (int, class) - the Tracker-ID for the next image
//              increasedTracker_ (boolean, class) - increase the Tracker-ID ?
//
  public Image getImage( String search )
  {
     return getImage( search, -1, 1, null); // transparent filtering!!!
  }
  public Image getImage( String search, int trackID )
  {
     return getImage( search, trackID, 1, null); // transparent filtering!!!
  }
  public Image getImage( String search, int trackID, int fltrType )
  {
    return getImage( search, trackID, fltrType, null ); // no filtering!!!
  }

  public Image getImage (String search, int trackID, int fltrType, ImageFilter fltr)
  {
    ImageReference tmpref;
    Image currentImg = null;
    Image tmpimg  = null;
    URL currentURL = null;

    int tracknum = -1;
    int i;

    // System.err.println("Searching for: " + search);

    if (search == null || res_ == null)  return null;

    tmpref = res_.searchImageReference( fltrType+"_"+search, false, true );

    if ( tmpref == null )
    {
      try
      {
        if ( fltrType < 100 )
        {
          if ( baseurl_ != null )
            currentURL = new URL( baseurl_, search );
          else
            currentURL = new URL( search );

          if ( app == null )
          {
            Toolkit toolkit = Toolkit.getDefaultToolkit();
            currentImg = toolkit.getImage( currentURL );
          }
          else
          {
            currentImg = app.getImage( currentURL );
          }
          i= fltrType;
        } else {
          currentImg = getImage( search, trackID, fltrType/100, null );
        }

        if ( currentImg != null )
        {
//          ImageReference newr= null;
//          newr.getName();

          switch( fltrType%100 )
          {
            case   1: fltr = new TransparencyFilter();
                      break;

            case   2: fltr = new GrayFilter();
                      break;

            case   3: fltr = new LightenFilter();
                      break;

            case   4: fltr = new DarkenFilter();
                      break;

            case  20: fltr = new BackGrayFilter();
                      break;

            case 0:
            default: // and zero -> no new pic -> set temporary ref
              tmpimg = currentImg;

          } // end of switchFilter

          if ( fltr != null )
            tmpimg = doFilter( currentImg, fltr, app );

          if (tmpimg != null) 
          {
            if (tracker_ != null) 
            {
              if ( trackID == -1 ) 
              {
                trackID = currentTID_;
                if ( increasedTracker_ )  currentTID_++;
              }

              tracker_.addImage( tmpimg, trackID );
              if (checkList_ != null)
                checkList_.insert( trackID, search+ ", Filter #"+fltrType );
              else 
                checkList_ = new StoreList( trackID, search );
            }

            tmpref = new ImageReference( fltrType+"_"+search, tmpimg, myobs_ );
            res_.addImageReference( tmpref );

//            if ( DEBUG )
//              System.out.println( "Searched: >" + search + "< (Filter: "+fltrType+") created ....>"+tmpimg+"<");
          }
        }
      }
      catch (MalformedURLException e)
      {
        System.err.println("Image >"+search+"< has a malformed URL:" + e.getMessage());
        // e1.printStackTrace();
        return null;
      }
      catch (IOException e) 
      {
        System.err.println("Image >"+search+"< could not be created due to an IOException : " + e.getMessage());
        return null;
      } 
      catch (NullPointerException e)
      {
        System.err.println("Image >"+search+"< could not be created due to NullPointer-Exception: " + e.getMessage());
        e.printStackTrace();
        return null;
      }
      catch (SecurityException e) 
      {
        System.err.println("Image >"+search+"< could not be created due to Security-Exception:" + e.getMessage());
        return null;
      }

      if ( tmpimg == null )   // is there a new 'filtered' image?
      {
/*
        if ( DEBUG && fltrType != 0) 
        {
          System.out.println("Warning: a certain image could not be filtered for proper displaying in the Simulator!");
          System.out.println("Imagename: " + search);
        }
*/
        if ( currentImg == null )
          return null;
        else
          tmpimg = currentImg;              // No? then use the current...
      }
    } else {     // -> a proper image was found in the current image storage
      tmpimg = tmpref.getImage();
    }

    return tmpimg;
  }


// Method: checkLoading (public)
//         checkTracker (public)
//
// *Purpose   : checks the loading of all  Image-Rsources
//
// *Parameters: baseurl (URL/String) - the basic url of the current browser
//              newimg (Image) - the new image
//              newobs (ImageObserver) - a new observer for the image
//
// *Return    : boolean - was the loading successful ?
//
// *Variables : tracker_ (MediaTracker, class) - for tracking the loading
//              checkList_ (StoreList, class) - stores all the unchecked images
//
  public boolean checkLoading()
  {
    boolean ret  = false;
    boolean ret2 = false;
    int i= 0;

    if (checkList_ == null)  return true;

    if ( tracker_ != null ) // no tracker no check!
    {
 //     tracker_.checkAll( true );

//      if ( !tracker_.isErrorAny() )
//        ret = true;

      for ( i = 0; (i < MAX_TRY) && !ret; i++ )
      {
        if ( !(ret = checkTracker()) )  //&& DEBUG)
          if (i < (MAX_TRY-1))
            System.err.println( "Retrying to load failed images... try #" + (i+2) + " ...");
          else
            System.err.println( "Giving up checking of images .. failed tries: "+i+"... ");
      }
    }

    if (checkList_ != null)
      checkList_.printList();

    return ret; // can't do more...
  }

  public boolean checkTracker()
  {
    int tracknum;
    boolean ret = true;

    if ( tracker_ == null ||
         checkList_ == null ) return true; // no tracker no check!

    checkList_ = checkList_.firstElement();
    StoreList tmplist = checkList_;
    LinList   namelist;

    while ( tmplist != null ) 
    {
      tracknum = tmplist.getData();
      try
      {
        tracker_.waitForID( tracknum );

/*
        if ( DEBUG )
        {
          namelist= tmplist.getList();
          if (namelist != null) {
            if (namelist.nextElement() == null) {
              System.out.println("Checking loading image: " + 
                namelist.getData() + ",  Track-ID: " + tracknum);
            }
            else {
              System.out.println("Checking loading at least one of these images:");

              while ( namelist != null ) {
                System.out.println("   >" + namelist.getData() + "<");
                namelist= namelist.nextElement();
              }
              System.out.println("Impossible to determine which may failed, " +
                "because all are checked with a single Track-ID #" +tracknum);
            }
          }
          else
            System.out.println("Checking loading of at least one image " + 
              ", Track-ID #" + tracknum );
        }
*/
      }
      catch (InterruptedException e) 
      {
        System.err.println("Interrupted Loading an Image: " + e.getMessage());
        ret = false;
      }

      // Check for errors loading it.
      if ( tracker_.isErrorID( tracknum ) ) 
      {
        namelist= tmplist.getList();
        if (namelist != null) {
          if (namelist.nextElement() == null) {
            System.err.println("Error loading image: " + 
              namelist.getData() + "");
          }
          else {
            System.err.println("Error loading at least one of these images:");

            while ( namelist != null ) {
              System.err.println("   >" + namelist.getData() + "<");
              namelist= namelist.nextElement();
            }
            System.err.println("Impossible to determine which one failed, " +
              " because they are all loaded in a single package.");
          }
        }
        else
          System.err.println("Error loading at least one image " + 
            "(Track-ID #" + tracknum + ")");

        ret = false;
        tmplist = tmplist.nextElement(); 
      } 
      else {
        if (tmplist == checkList_) {
          checkList_ = checkList_.unlinkNode();
          tmplist = checkList_; 
        } else
          tmplist = tmplist.unlinkNode();
      }
    }

    return ret;
  }

  Image getLoadingImage()
  {
    String tmpstr = props_.getLinkableProperty( "images.LOADING", null);

    if (tmpstr == null)
      return null;

    return getImage( tmpstr, 0, 0, null );
  }

}// End of class ResourceLoader
